import numpy as np
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import arviz as az
import csv
import pickle

rng = np.random.default_rng()

with open('raw_data.pkl', 'rb') as f:
    raw_data = pickle.load(f)
with open('N_st.pkl', 'rb') as f:
    N_st = pickle.load(f)

"""
        データ（i, j, s, t, N, TN）の設定
"""
N_data = 0
ID_i = []
ID_j = []
ID_s = []
ID_t = []
X = []
for v in raw_data[1:]:
    N_data += 1
    ID_i.append(int(v[1]))
    ID_j.append(int(v[2]))
    ID_s.append(int(v[3]))
    ID_t.append(int(v[4]))
    X.append(int(v[5]))

print('N_data = ', N_data)

for i in range(N_data):
    print("{0:>3}: {1}, {2}, {3}, {4}/  {5}".
          format((i + 1), ID_i[i], ID_j[i], ID_s[i], ID_t[i], X[i]))
    
with open('inf_data.pkl', 'rb') as f:
    inf_data = pickle.load(f)
    
print(az.summary(inf_data, var_names=['theta', 'sgm']))

az.plot_trace(inf_data,  var_names=['theta',  'sgm'],
              figsize=(12,7))  #, 'psi', 'sgm'])  #, var_names=['psi'])
plt.tight_layout()
plt.show()

with open('df_smpl.pkl', 'rb') as f:
    df_smpl = pickle.load(f)
print(df_smpl)

"""
        ψの事後分布
"""
Psi = [0]*N_st
for i in range(N_st):
    Psi[i] = df_smpl[f'psi.{i+1}'].values
for i in range(N_st):
    print(Psi[i])

plt.plot([0, 0], [0, 10], label = r'$\psi1$')   #    ψ0=0 に固定
for i in range(1, N_st-1):
    sb.kdeplot(Psi[i], label = r'$\psi${}'.format(i+1))
plt.plot([1, 1], [0, 10], label = r'$\psi${}'.format(N_st))      #   ψN_st=1 に固定
plt.legend()
plt.yticks([])
plt.xlabel(r'$\psi$', fontsize = 16)
plt.title('Posterior Distributions', fontsize = 18)
plt.savefig('FigPsi.png')
plt.show()

"""
         σの事後分布
"""
Sgm = df_smpl['sgm']  
sb.kdeplot(Sgm)
plt.xlabel(r'$\sigma$', fontsize = 14)
plt.title(r'Posterir distribution of $\sigma$', fontsize = 18)
plt.yticks([])
plt.savefig('FigSgm.png')
plt.show()

Sgm_med = np.percentile(Sgm, 50)
print('Sgm_med =', Sgm_med)

Psi_med = []
Psi_025 = []
Psi_25 = []
Psi_75 = []
Psi_975 = []

for i in range(N_st):
    q025, q25, med, q75, q975 = np.percentile(Psi[i], [2.5, 25, 50, 75, 97.5])
    Psi_med.append(med)
    Psi_025.append(q025)
    Psi_25.append(q25)
    Psi_75.append(q75)
    Psi_975.append(q975)
    
stimuli = []
for i in range(N_st):
    stimuli.append(i + 1)
"""
      刺激値と感覚量の関係
"""
plt.plot(stimuli, Psi_med, c='b', marker='o', lw=0, label = 'Median')
plt.plot(stimuli, Psi_25, c='g', marker='o', fillstyle='none', lw=0, label = '50% CI')
plt.plot(stimuli, Psi_75, c='g', marker='o', fillstyle='none', lw=0)
plt.plot(stimuli, Psi_025, c='y', marker='o', fillstyle='none', lw=0, label = '95% CI')
plt.plot(stimuli, Psi_975, c='y', marker='o', fillstyle='none', lw=0)
plt.plot([stimuli[0], stimuli[N_st-1]], [0, 1], c='r')
plt.title('Sensation and Stimulus', fontsize = 18)
plt.xticks(stimuli, fontsize=14)
plt.xlabel('Stimulus ID', fontsize = 14)
plt.ylabel(r'Sensation $\psi$', fontsize = 14)
plt.legend()  
plt.tight_layout()
plt.savefig('FigSummary.png')
plt.show()

Prob_model = np.empty(N_data)
for i in range(N_data):
    Prob_model[i] = ss.norm(0, 1).cdf((abs(Psi_med[ID_i[i]-1] - Psi_med[ID_j[i]-1]) -
                                   abs(Psi_med[ID_s[i]-1] - Psi_med[ID_t[i]-1])) / Sgm_med)
"""
       データ値とモデル値の関係
"""
for i in range(N_data):
    plt.plot(Prob_model[i], #+rng.uniform(-0.1,0.1),
             X[i]+rng.uniform(-0.1,0.1), 'o', fillstyle='none')
plt.title('Scattergram of (Prob_Model, Resp+jitter)', fontsize = 14)
plt.ylabel('Resp', fontsize = 14)
plt.yticks([0, 1], fontsize=14)
plt.xlabel('Prob.', fontsize = 16)
plt.tight_layout()
plt.show()
